@@ -1,8 +1,7 @@
 # Copyright: 2017, Ansible Project
 # Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause )
 
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 from ansible.module_utils.six import binary_type, text_type
 from ansible.module_utils.common.text.converters import to_text
