@@ -55,14 +55,14 @@ options:
   state:
     description:
       - Whether to install (V(present), V(latest)), or remove (V(absent)) a package.
-      - Default is V(None), however in effect the default action is V(present) unless the O(autoremove) option is
-        enabled for this module, then V(absent) is inferred.
+      - Default is V(None), however in effect the default action is V(present) unless the O(autoremove=true),
+        then V(absent) is inferred.
     choices: ['absent', 'present', 'installed', 'removed', 'latest']
     type: str
 
   enablerepo:
     description:
-      - I(Repoid) of repositories to enable for the install/update operation.
+      - C(Repoid) of repositories to enable for the install/update operation.
         These repos will not persist beyond the transaction.
         When specifying multiple repos, separate them with a ",".
     type: list
@@ -71,9 +71,9 @@ options:
 
   disablerepo:
     description:
-      - I(Repoid) of repositories to disable for the install/update operation.
+      - C(Repoid) of repositories to disable for the install/update operation.
         These repos will not persist beyond the transaction.
-        When specifying multiple repos, separate them with a ",".
+        When specifying multiple repos, separate them with a C(,).
     type: list
     elements: str
     default: []
@@ -86,7 +86,7 @@ options:
   disable_gpg_check:
     description:
       - Whether to disable the GPG checking of signatures of packages being
-        installed. Has an effect only if O(state) is V(present) or V(latest).
+        installed. Has an effect only if O(state=present) or O(state=latest).
       - This setting affects packages installed from a repository as well as
         "local" packages installed from the filesystem or a URL.
     type: bool
@@ -111,13 +111,13 @@ options:
     description:
       - If V(true), removes all "leaf" packages from the system that were originally
         installed as dependencies of user-installed packages but which are no longer
-        required by any such package. Should be used alone or when O(state) is V(absent)
+        required by any such package. Should be used alone or when O(state=absent).
     type: bool
     default: "no"
     version_added: "2.4"
   exclude:
     description:
-      - Package name(s) to exclude when state=present, or latest. This can be a
+      - Package name(s) to exclude when O(state=present), or latest. This can be a
         list or a comma separated string.
     version_added: "2.7"
     type: list
@@ -126,14 +126,14 @@ options:
   skip_broken:
     description:
       - Skip all unavailable packages or packages with broken dependencies
-        without raising an error. Equivalent to passing the --skip-broken option.
+        without raising an error. Equivalent to passing the C(--skip-broken) option.
     type: bool
     default: "no"
     version_added: "2.7"
   update_cache:
     description:
       - Force dnf to check if cache is out of date and redownload if needed.
-        Has an effect only if O(state) is V(present) or V(latest).
+        Has an effect only if O(state=present) or O(state=latest).
     type: bool
     default: "no"
     aliases: [ expire-cache ]
@@ -141,7 +141,7 @@ options:
   update_only:
     description:
       - When using latest, only update installed packages. Do not install packages.
-      - Has an effect only if O(state) is V(latest)
+      - Has an effect only if O(state=present) or O(state=latest).
     default: "no"
     type: bool
     version_added: "2.7"
@@ -161,7 +161,7 @@ options:
     version_added: "2.7"
   enable_plugin:
     description:
-      - I(Plugin) name to enable for the install/update operation.
+      - C(Plugin) name to enable for the install/update operation.
         The enabled plugin will not persist beyond the transaction.
     version_added: "2.7"
     type: list
@@ -169,7 +169,7 @@ options:
     default: []
   disable_plugin:
     description:
-      - I(Plugin) name to disable for the install/update operation.
+      - C(Plugin) name to disable for the install/update operation.
         The disabled plugins will not persist beyond the transaction.
     version_added: "2.7"
     type: list
@@ -179,13 +179,14 @@ options:
     description:
       - Disable the excludes defined in DNF config files.
       - If set to V(all), disables all excludes.
-      - If set to V(main), disable excludes defined in [main] in dnf.conf.
+      - If set to V(main), disable excludes defined in C([main]) in C(dnf.conf).
       - If set to V(repoid), disable excludes defined for given repo id.
     version_added: "2.7"
     type: str
   validate_certs:
     description:
-      - This only applies if using a https url as the source of the rpm. e.g. for localinstall. If set to V(false), the SSL certificates will not be validated.
+      - This only applies if using a https url as the source of the rpm. For example, for localinstall.
+        If set to V(false), the SSL certificates will not be validated.
       - This should only set to V(false) used on personally controlled sites using self-signed certificates as it avoids verifying the source site.
     type: bool
     default: "yes"
@@ -201,7 +202,7 @@ options:
     description:
       - Specify if the named package and version is allowed to downgrade
         a maybe already installed higher version of that package.
-        Note that setting allow_downgrade=True can make this module
+        Note that setting O(allow_downgrade=true) can make this module
         behave in a non-idempotent way. The task could end up with a set
         of packages that does not match the complete list of specified
         packages to install (because dependencies between the downgraded
@@ -244,7 +245,7 @@ options:
     version_added: "2.8"
   allowerasing:
     description:
-      - If V(true) it allows  erasing  of  installed  packages to resolve dependencies.
+      - If V(true) it allows erasing of installed packages to resolve dependencies.
     required: false
     type: bool
     default: "no"
@@ -288,20 +289,18 @@ attributes:
     platform:
         platforms: rhel
 notes:
-  - When used with a C(loop:) each package will be processed individually, it is much more efficient to pass the list directly to the I(name) option.
+  - When used with a C(loop:) each package will be processed individually, it is much more efficient to pass the list directly to the O(name) option.
   - Group removal doesn't work if the group was installed with Ansible because
     upstream dnf's API doesn't properly mark groups as installed, therefore upon
     removal the module is unable to detect that the group is installed
-    (https://bugzilla.redhat.com/show_bug.cgi?id=1620324)
+    U(https://bugzilla.redhat.com/show_bug.cgi?id=1620324).
   - While O(use_backend=yum) and the ability to call the action plugin as
     M(ansible.builtin.yum) are provided for syntax compatibility, the YUM
     backend was removed in ansible-core 2.17 because the required libraries are
     not available for any supported version of Python. If you rely on this
     functionality, use an older version of Ansible.
 requirements:
-  - "python >= 2.6"
-  - python-dnf
-  - for the autoremove option you need dnf >= 2.0.1"
+  - python3-dnf
 author:
   - Igor Gnatenko (@ignatenkobrain) <i.gnatenko.brain@gmail.com>
   - Cristian van Ee (@DJMuggs) <cristian at cvee.org>
@@ -402,7 +401,6 @@ import sys
 
 from ansible.module_utils.common.text.converters import to_native, to_text
 from ansible.module_utils.urls import fetch_file
-from ansible.module_utils.compat.version import LooseVersion
 
 from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils.common.locale import get_best_parsable_locale
@@ -428,11 +426,7 @@ class DnfModule(YumDnf):
 
         self._ensure_dnf()
         self.pkg_mgr_name = "dnf"
-
-        try:
-            self.with_modules = dnf.base.WITH_MODULES
-        except AttributeError:
-            self.with_modules = False
+        self.with_modules = dnf.base.WITH_MODULES
 
     def _sanitize_dnf_error_msg_install(self, spec, error):
         """
@@ -447,22 +441,6 @@ class DnfModule(YumDnf):
 
         return error
 
-    def _sanitize_dnf_error_msg_remove(self, spec, error):
-        """
-        For unhandled dnf.exceptions.Error scenarios, there are certain error
-        messages we want to ignore in a removal scenario as known benign
-        failures. Do that here.
-        """
-        if (
-            'no package matched' in to_native(error) or
-            'No match for argument:' in to_native(error)
-        ):
-            return (False, "{0} is not installed".format(spec))
-
-        # Return value is tuple of:
-        #   ("Is this actually a failure?", "Error Message")
-        return (True, error)
-
     def _package_dict(self, package):
         """Return a dictionary of information for the package."""
         # NOTE: This no longer contains the 'dnfstate' field because it is
@@ -511,7 +489,6 @@ class DnfModule(YumDnf):
 
         system_interpreters = ['/usr/libexec/platform-python',
                                '/usr/bin/python3',
-                               '/usr/bin/python2',
                                '/usr/bin/python']
 
         if not has_respawned():
@@ -526,7 +503,7 @@ class DnfModule(YumDnf):
         # done all we can do, something is just broken (auto-install isn't useful anymore with respawn, so it was removed)
         self.module.fail_json(
             msg="Could not import the dnf python module using {0} ({1}). "
-                "Please install `python3-dnf` or `python2-dnf` package or ensure you have specified the "
+                "Please install `python3-dnf` package or ensure you have specified the "
                 "correct ansible_python_interpreter. (attempted {2})"
                 .format(sys.executable, sys.version.replace('\n', ''), system_interpreters),
             results=[]
@@ -603,6 +580,11 @@ class DnfModule(YumDnf):
             # setting this to an empty string instead of None appears to mimic the DNF CLI behavior
             conf.substitutions['releasever'] = ''
 
+        # Honor installroot for dnf directories
+        # This will also perform variable substitutions in the paths
+        for opt in ('cachedir', 'logdir', 'persistdir'):
+            conf.prepend_installroot(opt)
+
         # Set skip_broken (in dnf this is strict=0)
         if self.skip_broken:
             conf.strict = 0
@@ -653,22 +635,14 @@ class DnfModule(YumDnf):
         """Return a fully configured dnf Base object."""
         base = dnf.Base()
         self._configure_base(base, conf_file, disable_gpg_check, installroot, sslverify)
-        try:
-            # this method has been supported in dnf-4.2.17-6 or later
-            # https://bugzilla.redhat.com/show_bug.cgi?id=1788212
-            base.setup_loggers()
-        except AttributeError:
-            pass
-        try:
-            base.init_plugins(set(self.disable_plugin), set(self.enable_plugin))
-            base.pre_configure_plugins()
-        except AttributeError:
-            pass  # older versions of dnf didn't require this and don't have these methods
+
+        base.setup_loggers()
+        base.init_plugins(set(self.disable_plugin), set(self.enable_plugin))
+        base.pre_configure_plugins()
+
         self._specify_repositories(base, disablerepo, enablerepo)
-        try:
-            base.configure_plugins()
-        except AttributeError:
-            pass  # older versions of dnf didn't require this and don't have these methods
+
+        base.configure_plugins()
 
         try:
             if self.update_cache:
@@ -743,18 +717,25 @@ class DnfModule(YumDnf):
         else:
             return bool(installed_query)
 
-    def _is_newer_version_installed(self, pkg_name):
+    def _is_newer_version_installed(self, pkg_spec):
         try:
-            if isinstance(pkg_name, dnf.package.Package):
-                available = pkg_name
+            if isinstance(pkg_spec, dnf.package.Package):
+                installed = sorted(self.base.sack.query().installed().filter(name=pkg_spec.name, arch=pkg_spec.arch))[-1]
+                return installed.evr_gt(pkg_spec)
             else:
-                available = sorted(
-                    dnf.subject.Subject(pkg_name).get_best_query(sack=self.base.sack).available().run()
-                )[-1]
-            installed = sorted(self.base.sack.query().installed().filter(name=available.name).run())[-1]
+                available = dnf.subject.Subject(pkg_spec).get_best_query(sack=self.base.sack).available()
+                installed = self.base.sack.query().installed().filter(name=available[0].name)
+                for arch in sorted(set(p.arch for p in installed)):  # select only from already-installed arches for this case
+                    installed_pkg = sorted(installed.filter(arch=arch))[-1]
+                    try:
+                        available_pkg = sorted(available.filter(arch=arch))[-1]
+                    except IndexError:
+                        continue  # nothing currently available for this arch; keep going
+                    if installed_pkg.evr_gt(available_pkg):
+                        return True
+                return False
         except IndexError:
             return False
-        return installed > available
 
     def _mark_package_install(self, pkg_spec, upgrade=False):
         """Mark the package for install."""
@@ -813,16 +794,13 @@ class DnfModule(YumDnf):
                 "results": []
             }
         except dnf.exceptions.Error as e:
-            if to_text("already installed") in to_text(e):
-                return {'failed': False, 'msg': '', 'failure': ''}
-            else:
-                return {
-                    'failed': True,
-                    'msg': "Unknown Error occurred for package {0}.".format(pkg_spec),
-                    'failure': " ".join((pkg_spec, to_native(e))),
-                    'rc': 1,
-                    "results": []
-                }
+            return {
+                'failed': True,
+                'msg': "Unknown Error occurred for package {0}.".format(pkg_spec),
+                'failure': " ".join((pkg_spec, to_native(e))),
+                'rc': 1,
+                "results": []
+            }
 
         return {'failed': False, 'msg': msg, 'failure': '', 'rc': 0}
 
@@ -886,36 +864,20 @@ class DnfModule(YumDnf):
         return not_installed
 
     def _install_remote_rpms(self, filenames):
-        if int(dnf.__version__.split(".")[0]) >= 2:
-            pkgs = list(sorted(self.base.add_remote_rpms(list(filenames)), reverse=True))
-        else:
-            pkgs = []
-            try:
-                for filename in filenames:
-                    pkgs.append(self.base.add_remote_rpm(filename))
-            except IOError as e:
-                if to_text("Can not load RPM file") in to_text(e):
-                    self.module.fail_json(
-                        msg="Error occurred attempting remote rpm install of package: {0}. {1}".format(filename, to_native(e)),
-                        results=[],
-                        rc=1,
-                    )
-        if self.update_only:
-            self._update_only(pkgs)
-        else:
-            for pkg in pkgs:
-                try:
-                    if self._is_newer_version_installed(pkg):
-                        if self.allow_downgrade:
-                            self.base.package_install(pkg, strict=self.base.conf.strict)
-                    else:
+        try:
+            pkgs = self.base.add_remote_rpms(filenames)
+            if self.update_only:
+                self._update_only(pkgs)
+            else:
+                for pkg in pkgs:
+                    if not (self._is_newer_version_installed(pkg) and not self.allow_downgrade):
                         self.base.package_install(pkg, strict=self.base.conf.strict)
-                except Exception as e:
-                    self.module.fail_json(
-                        msg="Error occurred attempting remote rpm operation: {0}".format(to_native(e)),
-                        results=[],
-                        rc=1,
-                    )
+        except Exception as e:
+            self.module.fail_json(
+                msg="Error occurred attempting remote rpm operation: {0}".format(to_native(e)),
+                results=[],
+                rc=1,
+            )
 
     def _is_module_installed(self, module_spec):
         if self.with_modules:
@@ -932,7 +894,7 @@ class DnfModule(YumDnf):
                 else:
                     return True         # No stream provided, but module found
 
-        return False  # seems like a sane default
+        return False  # seems like a logical default
 
     def ensure(self):
 
@@ -1136,14 +1098,6 @@ class DnfModule(YumDnf):
                     except dnf.exceptions.CompsError:
                         # Group is already uninstalled.
                         pass
-                    except AttributeError:
-                        # Group either isn't installed or wasn't marked installed at install time
-                        # because of DNF bug
-                        #
-                        # This is necessary until the upstream dnf API bug is fixed where installing
-                        # a group via the dnf API doesn't actually mark the group as installed
-                        #   https://bugzilla.redhat.com/show_bug.cgi?id=1620324
-                        pass
 
                 for environment in environments:
                     try:
@@ -1152,25 +1106,11 @@ class DnfModule(YumDnf):
                         # Environment is already uninstalled.
                         pass
 
-                installed = self.base.sack.query().installed()
                 for pkg_spec in pkg_specs:
-                    # short-circuit installed check for wildcard matching
-                    if '*' in pkg_spec:
-                        try:
-                            self.base.remove(pkg_spec)
-                        except dnf.exceptions.MarkingError as e:
-                            is_failure, handled_remove_error = self._sanitize_dnf_error_msg_remove(pkg_spec, to_native(e))
-                            if is_failure:
-                                failure_response['failures'].append('{0} - {1}'.format(pkg_spec, to_native(e)))
-                            else:
-                                response['results'].append(handled_remove_error)
-                        continue
-
-                    installed_pkg = dnf.subject.Subject(pkg_spec).get_best_query(
-                        sack=self.base.sack).installed().run()
-
-                    for pkg in installed_pkg:
-                        self.base.remove(str(pkg))
+                    try:
+                        self.base.remove(pkg_spec)
+                    except dnf.exceptions.MarkingError as e:
+                        response['results'].append(f"{e.value}: {pkg_spec}")
 
                 # Like the dnf CLI we want to allow recursive removal of dependent
                 # packages
@@ -1224,10 +1164,8 @@ class DnfModule(YumDnf):
 
                     self.base.download_packages(self.base.transaction.install_set)
                 except dnf.exceptions.DownloadError as e:
-                    self.module.fail_json(
-                        msg="Failed to download packages: {0}".format(to_text(e)),
-                        results=[],
-                    )
+                    failure_response['msg'] = "Failed to download packages: {0}".format(to_native(e))
+                    self.module.fail_json(**failure_response)
 
                 # Validate GPG. This is NOT done in dnf.Base (it's done in the
                 # upstream CLI subclass of dnf.Base)
@@ -1268,33 +1206,10 @@ class DnfModule(YumDnf):
             failure_response['msg'] = "Depsolve Error occurred: {0}".format(to_native(e))
             self.module.fail_json(**failure_response)
         except dnf.exceptions.Error as e:
-            if to_text("already installed") in to_text(e):
-                response['changed'] = False
-                response['results'].append("Package already installed: {0}".format(to_native(e)))
-                self.module.exit_json(**response)
-            else:
-                failure_response['msg'] = "Unknown Error occurred: {0}".format(to_native(e))
-                self.module.fail_json(**failure_response)
+            failure_response['msg'] = "Unknown Error occurred: {0}".format(to_native(e))
+            self.module.fail_json(**failure_response)
 
     def run(self):
-        """The main function."""
-
-        # Check if autoremove is called correctly
-        if self.autoremove:
-            if LooseVersion(dnf.__version__) < LooseVersion('2.0.1'):
-                self.module.fail_json(
-                    msg="Autoremove requires dnf>=2.0.1. Current dnf version is %s" % dnf.__version__,
-                    results=[],
-                )
-
-        # Check if download_dir is called correctly
-        if self.download_dir:
-            if LooseVersion(dnf.__version__) < LooseVersion('2.6.2'):
-                self.module.fail_json(
-                    msg="download_dir requires dnf>=2.6.2. Current dnf version is %s" % dnf.__version__,
-                    results=[],
-                )
-
         if self.update_cache and not self.names and not self.list:
             self.base = self._base(
                 self.conf_file, self.disable_gpg_check, self.disablerepo,
