@@ -47,11 +47,6 @@ from ...ansible_util import (
     ansible_environment,
 )
 
-from ...python_requirements import (
-    PipUnavailableError,
-    install_requirements,
-)
-
 from ...config import (
     SanityConfig,
 )
@@ -68,10 +63,6 @@ from ...host_configs import (
     PythonConfig,
 )
 
-from ...venv import (
-    get_virtualenv_version,
-)
-
 
 def _get_module_test(module_restrictions: bool) -> c.Callable[[str], bool]:
     """Create a predicate which tests whether a path can be used by modules or not."""
@@ -109,15 +100,6 @@ class ImportTest(SanityMultipleVersion):
 
         paths = [target.path for target in targets.include]
 
-        if python.version.startswith('2.') and (get_virtualenv_version(args, python.path) or (0,)) < (13,):
-            # hack to make sure that virtualenv is available under Python 2.x
-            # on Python 3.x we can use the built-in venv
-            # version 13+ is required to use the `--no-wheel` option
-            try:
-                install_requirements(args, python, virtualenv=True, controller=False)  # sanity (import)
-            except PipUnavailableError as ex:
-                display.warning(str(ex))
-
         temp_root = os.path.join(ResultType.TMP.path, 'sanity', 'import')
 
         messages = []
