@@ -21,6 +21,7 @@ from ...util import (
     ANSIBLE_TEST_DATA_ROOT,
     display,
     is_subdir,
+    str_to_version,
     SubprocessError,
     ANSIBLE_LIB_ROOT,
     ANSIBLE_TEST_TARGET_ROOT,
@@ -87,12 +88,13 @@ from ...host_profiles import (
 
 class TestContext:
     """Contexts that unit tests run in based on the type of content."""
+
     controller = 'controller'
     modules = 'modules'
     module_utils = 'module_utils'
 
 
-def command_units(args):  # type: (UnitsConfig) -> None
+def command_units(args: UnitsConfig) -> None:
     """Run unit tests."""
     handle_layout_messages(data_context().content.unit_messages)
 
@@ -102,7 +104,7 @@ def command_units(args):  # type: (UnitsConfig) -> None
 
     paths = [target.path for target in include]
 
-    content_config = get_content_config()
+    content_config = get_content_config(args)
     supported_remote_python_versions = content_config.modules.python_versions
 
     if content_config.modules.controller_only:
@@ -127,8 +129,8 @@ def command_units(args):  # type: (UnitsConfig) -> None
     if not paths:
         raise AllTargetsSkipped()
 
-    targets = t.cast(t.List[PosixConfig], args.targets)
-    target_versions = {target.python.version: target for target in targets}  # type: t.Dict[str, PosixConfig]
+    targets = t.cast(list[PosixConfig], args.targets)
+    target_versions: dict[str, PosixConfig] = {target.python.version: target for target in targets}
     skipped_versions = args.host_settings.skipped_python_versions
     warn_versions = []
 
@@ -220,7 +222,7 @@ def command_units(args):  # type: (UnitsConfig) -> None
             display.warning("Skipping unit tests on Python %s because it could not be found." % version)
             continue
 
-        target_profiles = {profile.config.python.version: profile for profile in host_state.targets(PosixProfile)}  # type: t.Dict[str, PosixProfile]
+        target_profiles: dict[str, PosixProfile] = {profile.config.python.version: profile for profile in host_state.targets(PosixProfile)}
         target_profile = target_profiles[version]
 
         final_candidates = [(test_context, target_profile.python, paths, env) for test_context, paths, env in test_candidates]
@@ -235,28 +237,36 @@ def command_units(args):  # type: (UnitsConfig) -> None
         sys.exit()
 
     for test_context, python, paths, env in test_sets:
+        # When using pytest-mock, make sure that features introduced in Python 3.8 are available to older Python versions.
+        # This is done by enabling the mock_use_standalone_module feature, which forces use of mock even when unittest.mock is available.
+        # Later Python versions have not introduced additional unittest.mock features, so use of mock is not needed as of Python 3.8.
+        # If future Python versions introduce new unittest.mock features, they will not be available to older Python versions.
+        # Having the cutoff at Python 3.8 also eases packaging of ansible-core since no supported controller version requires the use of mock.
+        #
+        # NOTE: This only affects use of pytest-mock.
+        #       Collection unit tests may directly import mock, which will be provided by ansible-test when it installs requirements using pip.
+        #       Although mock is available for ansible-core unit tests, they should import unittest.mock instead.
+        if str_to_version(python.version) < (3, 8):
+            config_name = 'legacy.ini'
+        else:
+            config_name = 'default.ini'
+
         cmd = [
             'pytest',
-            '--forked',
             '-r', 'a',
             '-n', str(args.num_workers) if args.num_workers else 'auto',
-            '--color',
-            'yes' if args.color else 'no',
+            '--color', 'yes' if args.color else 'no',
             '-p', 'no:cacheprovider',
-            '-c', os.path.join(ANSIBLE_TEST_DATA_ROOT, 'pytest.ini'),
+            '-c', os.path.join(ANSIBLE_TEST_DATA_ROOT, 'pytest', 'config', config_name),
             '--junit-xml', os.path.join(ResultType.JUNIT.path, 'python%s-%s-units.xml' % (python.version, test_context)),
-        ]
+            '--strict-markers',  # added in pytest 4.5.0
+            '--rootdir', data_context().content.root,
+            '--confcutdir', data_context().content.root,  # avoid permission errors when running from an installed version and using pytest >= 8
+        ]  # fmt:skip
 
         if not data_context().content.collection:
             cmd.append('--durations=25')
 
-        if python.version == '2.6':
-            # same as --strict-markers in older versions of pytest which still support python 2.6
-            cmd.append('--strict')
-        #else:
-        #    # added in pytest 4.5.0, which requires python 2.7+
-        #    cmd.append('--strict-markers')
-
         plugins = []
 
         if args.coverage:
@@ -265,6 +275,8 @@ def command_units(args):  # type: (UnitsConfig) -> None
         if data_context().content.collection:
             plugins.append('ansible_pytest_collections')
 
+        plugins.append('ansible_forked')
+
         if plugins:
             env['PYTHONPATH'] += ':%s' % os.path.join(ANSIBLE_TEST_TARGET_ROOT, 'pytest/plugins')
             env['PYTEST_PLUGINS'] = ','.join(plugins)
@@ -280,14 +292,14 @@ def command_units(args):  # type: (UnitsConfig) -> None
         display.info('Unit test %s with Python %s' % (test_context, python.version))
 
         try:
-            cover_python(args, python, cmd, test_context, env)
+            cover_python(args, python, cmd, test_context, env, capture=False)
         except SubprocessError as ex:
             # pytest exits with status code 5 when all tests are skipped, which isn't an error for our use case
             if ex.status != 5:
                 raise
 
 
-def get_units_ansible_python_path(args, test_context):  # type: (UnitsConfig, str) -> str
+def get_units_ansible_python_path(args: UnitsConfig, test_context: str) -> str:
     """
     Return a directory usable for PYTHONPATH, containing only the modules and module_utils portion of the ansible package.
     The temporary directory created will be cached for the lifetime of the process and cleaned up at exit.
@@ -296,9 +308,9 @@ def get_units_ansible_python_path(args, test_context):  # type: (UnitsConfig, st
         return get_ansible_python_path(args)
 
     try:
-        cache = get_units_ansible_python_path.cache
+        cache = get_units_ansible_python_path.cache  # type: ignore[attr-defined]
     except AttributeError:
-        cache = get_units_ansible_python_path.cache = {}
+        cache = get_units_ansible_python_path.cache = {}  # type: ignore[attr-defined]
 
     python_path = cache.get(test_context)
 
@@ -324,7 +336,6 @@ def get_units_ansible_python_path(args, test_context):  # type: (UnitsConfig, st
         # legacy collection loader required by all python versions not supported by the controller
         write_text_file(os.path.join(ansible_test_path, '__init__.py'), '', True)
         write_text_file(os.path.join(ansible_test_path, '_internal', '__init__.py'), '', True)
-        os.symlink(os.path.join(ANSIBLE_TEST_TARGET_ROOT, 'legacy_collection_loader'), os.path.join(ansible_test_path, '_internal', 'legacy_collection_loader'))
     elif test_context == TestContext.modules:
         # only non-collection ansible module tests should have access to ansible built-in modules
         os.symlink(os.path.join(ANSIBLE_LIB_ROOT, 'modules'), os.path.join(ansible_path, 'modules'))
