@@ -1,10 +1,8 @@
 """High level functions for working with containers."""
 from __future__ import annotations
 
-import atexit
 import collections.abc as c
 import contextlib
-import enum
 import json
 import random
 import time
@@ -20,6 +18,7 @@ from .util import (
 )
 
 from .util_common import (
+    ExitHandler,
     named_temporary_file,
 )
 
@@ -46,6 +45,7 @@ from .docker_util import (
     get_docker_container_id,
     get_docker_host_ip,
     get_podman_host_ip,
+    get_session_container_name,
     require_docker,
     detect_host_properties,
 )
@@ -95,18 +95,12 @@ support_containers_mutex = threading.Lock()
 
 class HostType:
     """Enum representing the types of hosts involved in running tests."""
+
     origin = 'origin'
     control = 'control'
     managed = 'managed'
 
 
-class CleanupMode(enum.Enum):
-    """How container cleanup should be handled."""
-    YES = enum.auto()
-    NO = enum.auto()
-    INFO = enum.auto()
-
-
 def run_support_container(
     args: EnvironmentConfig,
     context: str,
@@ -115,8 +109,7 @@ def run_support_container(
     ports: list[int],
     aliases: t.Optional[list[str]] = None,
     start: bool = True,
-    allow_existing: bool = False,
-    cleanup: t.Optional[CleanupMode] = None,
+    cleanup: bool = True,
     cmd: t.Optional[list[str]] = None,
     env: t.Optional[dict[str, str]] = None,
     options: t.Optional[list[str]] = None,
@@ -126,6 +119,8 @@ def run_support_container(
     Start a container used to support tests, but not run them.
     Containers created this way will be accessible from tests.
     """
+    name = get_session_container_name(args, name)
+
     if args.prime_containers:
         docker_pull(args, image)
         return None
@@ -146,7 +141,7 @@ def run_support_container(
         if current_container_id:
             publish_ports = False  # publishing ports is pointless if already running in a docker container
 
-    options = (options or [])
+    options = options or []
 
     if start:
         options.append('-dt')  # the -t option is required to cause systemd in the container to log output to the console
@@ -163,46 +158,13 @@ def run_support_container(
 
     options.extend(['--ulimit', 'nofile=%s' % max_open_files])
 
-    support_container_id = None
-
-    if allow_existing:
-        try:
-            container = docker_inspect(args, name)
-        except ContainerNotFoundError:
-            container = None
-
-        if container:
-            support_container_id = container.id
-
-            if not container.running:
-                display.info('Ignoring existing "%s" container which is not running.' % name, verbosity=1)
-                support_container_id = None
-            elif not container.image:
-                display.info('Ignoring existing "%s" container which has the wrong image.' % name, verbosity=1)
-                support_container_id = None
-            elif publish_ports and not all(port and len(port) == 1 for port in [container.get_tcp_port(port) for port in ports]):
-                display.info('Ignoring existing "%s" container which does not have the required published ports.' % name, verbosity=1)
-                support_container_id = None
-
-            if not support_container_id:
-                docker_rm(args, name)
-
     if args.dev_systemd_debug:
         options.extend(('--env', 'SYSTEMD_LOG_LEVEL=debug'))
 
-    if support_container_id:
-        display.info('Using existing "%s" container.' % name)
-        running = True
-        existing = True
-    else:
-        display.info('Starting new "%s" container.' % name)
-        docker_pull(args, image)
-        support_container_id = run_container(args, image, name, options, create_only=not start, cmd=cmd)
-        running = start
-        existing = False
-
-    if cleanup is None:
-        cleanup = CleanupMode.INFO if existing else CleanupMode.YES
+    display.info('Starting new "%s" container.' % name)
+    docker_pull(args, image)
+    support_container_id = run_container(args, image, name, options, create_only=not start, cmd=cmd)
+    running = start
 
     descriptor = ContainerDescriptor(
         image,
@@ -213,7 +175,6 @@ def run_support_container(
         aliases,
         publish_ports,
         running,
-        existing,
         cleanup,
         env,
     )
@@ -223,7 +184,7 @@ def run_support_container(
             raise Exception(f'Container already defined: {name}')
 
         if not support_containers:
-            atexit.register(cleanup_containers, args)
+            ExitHandler.register(cleanup_containers, args)
 
         support_containers[name] = descriptor
 
@@ -331,10 +292,13 @@ def get_docker_preferred_network_name(args: EnvironmentConfig) -> t.Optional[str
         current_container_id = get_docker_container_id()
 
         if current_container_id:
-            # Make sure any additional containers we launch use the same network as the current container we're running in.
-            # This is needed when ansible-test is running in a container that is not connected to Docker's default network.
-            container = docker_inspect(args, current_container_id, always=True)
-            network = container.get_network_name()
+            try:
+                # Make sure any additional containers we launch use the same network as the current container we're running in.
+                # This is needed when ansible-test is running in a container that is not connected to Docker's default network.
+                container = docker_inspect(args, current_container_id, always=True)
+                network = container.get_network_name()
+            except ContainerNotFoundError:
+                display.warning('Unable to detect the network for the current container. Use the `--docker-network` option if containers are unreachable.')
 
     # The default docker behavior puts containers on the same network.
     # The default podman behavior puts containers on isolated networks which don't allow communication between containers or network disconnect.
@@ -378,6 +342,7 @@ def get_container_database(args: EnvironmentConfig) -> ContainerDatabase:
 
 class ContainerAccess:
     """Information needed for one test host to access a single container supporting tests."""
+
     def __init__(self, host_ip: str, names: list[str], ports: t.Optional[list[int]], forwards: t.Optional[dict[int, int]]) -> None:
         # if forwards is set
         #   this is where forwards are sent (it is the host that provides an indirect connection to the containers on alternate ports)
@@ -437,6 +402,7 @@ class ContainerAccess:
 
 class ContainerDatabase:
     """Database of running containers used to support tests."""
+
     def __init__(self, data: dict[str, dict[str, dict[str, ContainerAccess]]]) -> None:
         self.data = data
 
@@ -576,6 +542,7 @@ def create_container_database(args: EnvironmentConfig) -> ContainerDatabase:
 
 class SupportContainerContext:
     """Context object for tracking information relating to access of support containers."""
+
     def __init__(self, containers: ContainerDatabase, process: t.Optional[SshProcess]) -> None:
         self.containers = containers
         self.process = process
@@ -678,19 +645,20 @@ def create_support_container_context(
 
 class ContainerDescriptor:
     """Information about a support container."""
-    def __init__(self,
-                 image: str,
-                 context: str,
-                 name: str,
-                 container_id: str,
-                 ports: list[int],
-                 aliases: list[str],
-                 publish_ports: bool,
-                 running: bool,
-                 existing: bool,
-                 cleanup: CleanupMode,
-                 env: t.Optional[dict[str, str]],
-                 ) -> None:
+
+    def __init__(
+        self,
+        image: str,
+        context: str,
+        name: str,
+        container_id: str,
+        ports: list[int],
+        aliases: list[str],
+        publish_ports: bool,
+        running: bool,
+        cleanup: bool,
+        env: t.Optional[dict[str, str]],
+    ) -> None:
         self.image = image
         self.context = context
         self.name = name
@@ -699,7 +667,6 @@ class ContainerDescriptor:
         self.aliases = aliases
         self.publish_ports = publish_ports
         self.running = running
-        self.existing = existing
         self.cleanup = cleanup
         self.env = env
         self.details: t.Optional[SupportContainer] = None
@@ -757,23 +724,26 @@ class ContainerDescriptor:
 
 class SupportContainer:
     """Information about a running support container available for use by tests."""
-    def __init__(self,
-                 container: DockerInspect,
-                 container_ip: str,
-                 published_ports: dict[int, int],
-                 ) -> None:
+
+    def __init__(
+        self,
+        container: DockerInspect,
+        container_ip: str,
+        published_ports: dict[int, int],
+    ) -> None:
         self.container = container
         self.container_ip = container_ip
         self.published_ports = published_ports
 
 
-def wait_for_file(args: EnvironmentConfig,
-                  container_name: str,
-                  path: str,
-                  sleep: int,
-                  tries: int,
-                  check: t.Optional[c.Callable[[str], bool]] = None,
-                  ) -> str:
+def wait_for_file(
+    args: EnvironmentConfig,
+    container_name: str,
+    path: str,
+    sleep: int,
+    tries: int,
+    check: t.Optional[c.Callable[[str], bool]] = None,
+) -> str:
     """Wait for the specified file to become available in the requested container and return its contents."""
     display.info('Waiting for container "%s" to provide file: %s' % (container_name, path))
 
@@ -795,10 +765,8 @@ def wait_for_file(args: EnvironmentConfig,
 def cleanup_containers(args: EnvironmentConfig) -> None:
     """Clean up containers."""
     for container in support_containers.values():
-        if container.cleanup == CleanupMode.YES:
-            docker_rm(args, container.container_id)
-        elif container.cleanup == CleanupMode.INFO:
-            display.notice(f'Remember to run `{require_docker().command} rm -f {container.name}` when finished testing.')
+        if container.cleanup:
+            docker_rm(args, container.name)
 
 
 def create_hosts_entries(context: dict[str, ContainerAccess]) -> list[str]:
@@ -853,6 +821,7 @@ def create_container_hooks(
             """Clean up previously configured SSH port forwarding which was required by the specified target."""
             cleanup_ssh_ports(args, control_connections, '%s_hosts_restore.yml' % control_type, control_state, target, HostType.control)
             cleanup_ssh_ports(args, managed_connections, '%s_hosts_restore.yml' % managed_type, managed_state, target, HostType.managed)
+
     else:
         pre_target, post_target = None, None
 
